#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Oct 12 11:42:52 2017

@author: afalaize
"""

import pyphs as phs
import numpy as np
import sympy as sy

import os

from pyphs.misc.plots.multiplots import multiplot

from effective_wire_lenght import lc

from plots.config import plots_config_model0 as original_plots_config
import matplotlib.pyplot as plt
import matplotlib as mpl

from parameters import model0 as m0pars

plots_config = original_plots_config.copy()
plt.close('all')

subs = dict([(phs.Core.symbols(p), m0pars[p]) for p in m0pars])


def simulation_model_0(Pl_value):

    # ----------------------------------------------------------------------- #
    # init core object
    core = phs.Core(label='model_0')

    # ----------------------------------------------------------------------- #
    # storage
    x = core.symbols(['phiC', 'pM', 'qD'])

    Lc = core.symbols('Lc')
    Hphi = x[0]**2/(2*Lc)

    Mcda = core.symbols('Mcda')
    Hm = x[1]**2/(2*Mcda)

    Ksa = core.symbols('Ksa')
    Hk = Ksa*x[2]**2/2

    H = Hphi + Hm + Hk

    core.add_storages(x, H)

    # ----------------------------------------------------------------------- #
    # Dissipations
    w = core.symbols(['iC', 'dtqd'])

    Rc = core.symbols('Rc')
    Rsa = core.symbols('Rsa')

    z = [Rc*w[0], Rsa*w[1]]

    core.add_dissipations(w, z)

    # ----------------------------------------------------------------------- #
    # Port
    y, u = core.symbols(['iC', 'vI'])

    core.add_ports(u, y)

    # ----------------------------------------------------------------------- #
    # Port

    B = core.symbols('B')
    Ql, Pl, lc0 = core.symbols(['Ql', 'Pl', 'lc0'])

    BL = B*lc(x[2], lc0, Pl, Ql)

    # ----------------------------------------------------------------------- #
    # Structure

    core.set_Jxx([[0, -BL, 0],
                  [BL, 0, -1],
                  [0,  1,  0]])

    core.set_Jxw([[1, 0],
                  [0, 1],
                  [0, 0]])

    core.set_Jxy([[1],
                  [0],
                  [0]])

    # ----------------------------------------------------------------------- #
    # Parameters

    # Update Pl value
    subs[core.symbols('Pl')] = Pl_value

    # Update core parameters
    core.subs.update(subs)

    # ----------------------------------------------------------------------- #
    # Signal

    config_signal = {'fs': 96e3,
                     'tsig': 1e-1,
                     'tdeb': 0,
                     'tend': 0,
                     'f0': 100.,
                     'A': 50.,
                     'attack_ratio': 1.
                     }

    sig = phs.signalgenerator(which='sin', **config_signal)
    u = sig[:, np.newaxis]

    config = {'fs': config_signal['fs'],  # Sample rate (Hz)
              'grad': 'discret',  # In {'discret', 'theta', 'trapez'}
              'theta': 0.,  # Theta-scheme for the structure
              'split': True,  # split implicit from explicit part
              'maxit': 10,  # Max number of iterations for NL solvers
              'eps': 1e-10,  # Global numerical tolerance
              'pbar': True,
              'lang': 'c++'
              }

    simu = core.to_simulation(config=config)

    simu.init(u=u)

    simu.process()

    datax = list(simu.data.t())

    vI = list(simu.data.u(vslice=0))
    phiC = list(simu.data.x(vslice=0))
    iC = simu.data['y', :, 0]
    pM = list(simu.data.x(vslice=1))
    qD = list(simu.data.x(vslice=2))

    print(BL.subs(subs))
    if isinstance(BL.subs(subs), (float, int, sy.Float, sy.Integer)):
        BL_value = BL.subs(subs)

        def BL_lambda(x):
            return BL_value*np.ones_like(x)

    else:
        BL_lambda = phs.numerics.lambdify([x[2], ], BL, subs=subs,
                                          simplify=True, theano=False)

    Bl = simu.data.x(vslice=2, postprocess=BL_lambda)
    Ps = simu.data.ps()
    Pd = simu.data.pd()
    Psd = np.array(list(map(sum, zip(Ps, Pd))))
    dtE = simu.data.dtE(vslice=0, postprocess=lambda el: -el)

# uncomment below to simulate model 0
    datay = [vI, phiC, pM, qD, Bl, (dtE, Psd)]

# uncomment below to simulate model 2 and compare with model 0
#    datay = [vI, iC, pM, qD, Bl, (dtE, Psd)]

    return datax, datay


#%%
if __name__ == '__main__':

    Pls = [5., 0.]
    all_datay = list()

    for pl in Pls:
        datax, datay_pl = simulation_model_0(pl)
        all_datay.append(datay_pl)

    datay = all_datay[0]

    datay[1:-1] = list(zip(*[el[1:-1] for el in all_datay]))
    labels = [None, ]+[[r'$P_\ell={}$'.format(e) for e in Pls]]*4 + [(r'$-\frac{\delta \mathrm{E}}{\delta t}$',
                                                                      r'$\mathrm{P_S}+\mathrm{P_D}$')]
    plots_config['labels'] = labels

    ylabels = [r'$v_{\mathrm{I}}$ (V)',
               r'$\phi_{\mathrm{C}}$ (Wb)',
               r'$p_{\mathrm{M}}$ (kg.m/s)',
               r'$q_{\mathrm{D}}$ (m)',
               r'$B\,\ell_{\mathrm{C}}$ (T.m)',
               'Power (W)']

    plots_config['xlabel'] = r'Time $t$ (s)'

    plots_config['ylabels'] = ylabels

    d = original_plots_config.copy()
    figpath = d.pop('path')
    loc = d.pop('loc')
    linestyles = d.pop('linestyles')
    mpl.rcParams.update(d)

    path = os.path.join(figpath, 'simu_model0')
    plots_config['path'] = path
    plots_config['grid'] = original_plots_config['axes.grid']

    fig, axs = multiplot(datax, datay, **plots_config)
